"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.any = any;
exports.constant = constant;
exports.contains = contains;
exports.extendUniq = extendUniq;
exports.filter = filter;
exports.flatMap = flatMap;
exports.map = map;
exports.reduce = reduce;
exports.selectedValues = selectedValues;
exports.startsWith = startsWith;
function any(p, arr) {
  var len = arr.length;

  for (var i = 0; i < len; i += 1) {
    if (p(arr[i])) {
      return true;
    }
  }

  return false;
}

function constant(x) {
  return function () {
    return x;
  };
}

function contains(el, arr) {
  var len = arr.length;

  for (var i = 0; i < len; i += 1) {
    if (arr[i] === el) {
      return true;
    }
  }

  return false;
}

function extendUniq(a, b) {
  var len = b.length;

  for (var i = 0; i < len; i += 1) {
    if (!contains(b[i], a)) {
      a.push(b[i]);
    }
  }

  return a;
}

function filter(p, arr) {
  var filtered = [];
  var len = arr.length;

  for (var i = 0; i < len; i += 1) {
    if (p(arr[i])) {
      filtered.push(arr[i]);
    }
  }

  return filtered;
}

function flatMap(fn, arr) {
  return reduce(function (acc, el) {
    var collection = fn(el);
    var len = collection.length;

    for (var i = 0; i < len; i += 1) {
      acc.push(collection[i]);
    }

    return acc;
  }, [], arr);
}

function map(fn, arr) {
  var mapped = [];
  var len = arr.length;

  for (var i = 0; i < len; i += 1) {
    mapped.push(fn(arr[i]));
  }

  return mapped;
}

function reduce(fn, acc, arr) {
  var len = arr.length;

  for (var i = 0; i < len; i += 1) {
    acc = fn(acc, arr[i]);
  }

  return acc;
}

function selectedValues(select) {
  var options = select.selectedOptions || filter(function (option) {
    return option.selected;
  }, select.options);

  return map(function (option) {
    return option.value;
  }, options);
}

function startsWith(match, string) {
  return string.slice(0, match.length) === match;
}